datablock AudioProfile(spyWarpCloakSound)
{
	filename = "./playerWarpCloak.wav";
	description = AudioClosestQuiet3d;
	preload = true;
};

datablock AudioProfile(spyWarpCloakUISound)
{
	filename = "./playerWarpCloak.wav";
	description = Audio2d;
	preload = true;
};

datablock AudioProfile(spyWarpCloakRevengeSound)
{
	filename = "./tf2Revenge.wav";
	description = Audio2d;
	preload = true;
};

datablock ItemData(SpyFakeDeathWatchItem)
{
	category = "Weapon";  // Mission editor category
	className = "Weapon"; // For inventory system

	 // Basic Item Properties
	shapeFile = "./DeadRingerItem.dts";
	rotate = false;
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;

	//gui stuff
	uiName = "Dead Ringer";
	iconName = "./icon_dwatch";
	doColorShift = true;
	colorShiftColor = "1 1 1 1";

	 // Dynamic properties defined by the scripts
	image = SpyFakeDeathWatchImage;
	canDrop = true;
	
	variableStore = "cloakEnergy";
	variableInitial = 100;
	variableReplace = 1;
};

datablock ShapeBaseImageData(SpyFakeDeathWatchImage)
{
   // Basic Item properties
   shapeFile = "./DeadRingerHand.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 -1000";
   eyeOffset = "0.75 1.5 -0.7";
   rotation = eulerToMatrix( "0 0 0" );
   eyeRotation = eulerToMatrix( "10 0 30" );

   className = "WeaponImage";
   item = SpyFakeDeathWatchItem;

   //raise your arm up or not
   armReady = false;
   
   //cloak stats
   isCloakWatch = 1;
   minCloakEnergy = 100;
   cloakEnergyDrain = 100/224; //7 seconds
   cloakChargeRate = 0.2;
   cloakDamageFlicker = 0;
   
   cloakSound = spyWarpCloakUISound;
   uncloakSound = spyWarpCloakSound;

   doColorShift = true;
   colorShiftColor = SpyFakeDeathWatchItem.colorShiftColor;

   // Initial start up state
   stateName[0]                     = "Startup";
   stateTransitionOnTimeout[0]      = "CloakUse";
   stateWaitForTimeout[0]           = 0;
   stateTimeoutValue[0]             = "0.05";
   stateSound[0]                    = "";
   stateAllowImageChange[0]         = true;
   
   stateName[1]                     = "Ready";
   stateTransitionOnAmmo[1]         = "Cloaking";
   stateAllowImageChange[1]         = true;
   stateSequence[1]                 = "Ready";
   
   stateName[2]                     = "Cloaking";
   stateTransitionOnNoAmmo[2]       = "Uncloak";
   stateAllowImageChange[2]         = true;
   stateSequence[2]                 = "Cloaking";
   stateTimeoutValue[2]             = 1;
   stateTransitionOnTimeout[2]      = "Cloaked";
   
   stateName[3]                     = "Cloaked";
   stateTransitionOnNoAmmo[3]       = "UncloakTransition";
   stateAllowImageChange[3]         = true;
   stateSequence[3]                 = "cloaked";
   stateTimeoutValue[3]             = 6.00;
   stateWaitForTimeout[3]           = 0;
   stateTransitionOnTimeout[3]      = "Uncloak"; //you should run out of cloak here anyway
   
   stateName[4]                     = "Uncloak";
   stateScript[4]                   = "onUnCloak";
   stateAllowImageChange[4]         = true;
   stateSequence[4]                 = "Uncloaking";
   stateTimeoutValue[4]             = 1;
   stateTransitionOnTimeout[4]      = "EndCloak";
   
   stateName[5]                     = "EndCloak";
   stateScript[5]                   = "onEndCloak";
   stateTimeoutValue[5]             = 0.1;
   stateTransitionOnTimeout[5]      = "Drained";
   
   stateName[6]                     = "UncloakTransition";
   stateTimeoutValue[6]             = 0.01;
   stateTransitionOnTimeout[6]      = "Uncloak";
   
   stateName[7]                     = "Drained";
   stateSequence[7]                 = "uncharged";
   stateScript[7]                   = "onDrained";
   stateTimeoutValue[7]             = 0.05;
   stateTransitionOnTimeout[7]      = "Drained";
   stateTransitionOnAmmo[7]         = "SwitchReady";
   
   stateName[8]                     = "SwitchReady";
   stateScript[8]                   = "onSwitchReady";
   stateTimeoutValue[8]             = 0.05;
   stateTransitionOnTimeout[8]      = "Ready";
   
   stateName[0]                     = "CloakUse";
   stateScript[0]                   = "onCloakUse";
   stateTransitionOnTimeout[0]      = "CloakUse";
   stateTransitionOnAmmo[0]         = "Activate";
   stateWaitForTimeout[0]           = 0;
   stateTimeoutValue[0]             = "0.05";
   stateSound[0]                    = "";
   stateAllowImageChange[0]         = true;
   
   stateName[9]                     = "Activate";
   stateScript[9]                   = "onSwitchReady";
   stateSequence[9]                 = "activate";
   stateTimeoutValue[9]             = 0.5;
   stateTransitionOnTimeout[9]      = "Ready";
   stateAllowImageChange[9]         = true;
   stateSound[9]                    = "";
};

function SpyFakeDeathWatchImage::onMount(%this,%obj,%slot)
{
   Parent::onMount(%this,%obj,%slot);
   
   %obj.setImageAmmo(%slot,0);
   fixArmReady(%obj);
   
   %obj.cloakEnergy = %obj.cloakEnergy + 20*%this.cloakChargeRate*(getSimTime()-%obj.lastUnCloakTime)/1000;
   %obj.lastUnCloakTime = getSimTime();
   
   if(%obj.cloakEnergy > 100) %obj.cloakEnergy = 100;
   if(%obj.cloakEnergy < 0) %obj.cloakEnergy = 0;
   
   if(isObject(%obj.client))
   {
      %a = %obj.cloakEnergy;
      %b = 100;
      %start = "<color:FFFFFF>";
      %sep = "<color:111111>";
      %done = (%a >= %b) || (%a == 0);
      %end = (%a / %b) * 25;
      %str = (%a == 0 ? %sep : %start);
      
      for(%i=0;%i<25;%i++)
      {
         if(%i > %end && !%done)
         {
            %str = %str @ %sep;
            %done = 1;
         }
         %str = %str @ "|";
      }
      bottomprint(%obj.client, "<just:center>" @ %str, 2, 1);
   }
}

function SpyFakeDeathWatchImage::onCloakUse(%this,%obj,%slot)
{
   //put arm down since it's invisible from outside
   fixArmReady(%obj);
   
   %obj.cloakEnergy = %obj.cloakEnergy + 20*%this.cloakChargeRate*(getSimTime()-%obj.lastUnCloakTime)/1000;
   %obj.lastUnCloakTime = getSimTime();
   
   if(%obj.cloakEnergy > 100) %obj.cloakEnergy = 100;
   if(%obj.cloakEnergy < 0) %obj.cloakEnergy = 0;
   
   if(isObject(%obj.client))
   {
      %a = %obj.cloakEnergy;
      %b = 100;
      %start = "<color:FFFFFF>";
      %sep = "<color:111111>";
      %done = (%a >= %b) || (%a == 0);
      %end = (%a / %b) * 25;
      %str = (%a == 0 ? %sep : %start);
      
      for(%i=0;%i<25;%i++)
      {
         if(%i > %end && !%done)
         {
            %str = %str @ %sep;
            %done = 1;
         }
         %str = %str @ "|";
      }
      bottomprint(%obj.client, "<just:center>" @ %str, 2, 1);
   }
   
   if(!%obj.isCloaked && !%obj.isCloaking && %obj.cloakEnergy >= %this.minCloakEnergy)
      %obj.setImageAmmo(%slot,1);
}

function SpyFakeDeathWatchImage::onUnCloak(%this,%obj,%slot)
{
   if(%obj.isCloaked)
   {
      %obj.endCloak();
      %obj.cloakEnergy = 0;
      if(isObject(%obj.client))
      {
         %a = %obj.cloakEnergy;
         %b = 100;
         %start = "<color:FFFFFF>";
         %sep = "<color:111111>";
         %done = (%a >= %b) || (%a == 0);
         %end = (%a / %b) * 25;
         %str = (%a == 0 ? %sep : %start);
         
         for(%i=0;%i<25;%i++)
         {
            if(%i > %end && !%done)
            {
               %str = %str @ %sep;
               %done = 1;
            }
            %str = %str @ "|";
         }
         bottomprint(%obj.client, "<just:center>" @ %str, 2, 1);
      }
   }
}

function SpyFakeDeathWatchImage::onDrained(%this,%obj,%slot)
{
   if(%obj.lastUnCloakTime > 0)
   {
      %obj.cloakEnergy = %obj.cloakEnergy + 20*%this.cloakChargeRate*(getSimTime()-%obj.lastUnCloakTime)/1000;
      %obj.lastUnCloakTime = getSimTime();
   }
   
   if(%obj.cloakEnergy > 100) %obj.cloakEnergy = 100;
   if(%obj.cloakEnergy < 0) %obj.cloakEnergy = 0;
   
   if(isObject(%obj.client))
   {
      %a = %obj.cloakEnergy;
      %b = 100;
      %start = "<color:FFFFFF>";
      %sep = "<color:111111>";
      %done = (%a >= %b) || (%a == 0);
      %end = (%a / %b) * 25;
      %str = (%a == 0 ? %sep : %start);
      
      for(%i=0;%i<25;%i++)
      {
         if(%i > %end && !%done)
         {
            %str = %str @ %sep;
            %done = 1;
         }
         %str = %str @ "|";
      }
      bottomprint(%obj.client, "<just:center>" @ %str, 2, 1);
   }
   
   if(%obj.cloakEnergy == 100)
      %obj.setImageAmmo(%slot,1);
}

function SpyFakeDeathWatchImage::onSwitchReady(%this,%obj,%slot)
{
   %obj.setImageAmmo(%slot,0);
}

package FakeDeathWatch
{
   function Armor::Damage(%this,%obj,%source,%pos,%amm,%type)
   {
      if(isObject(%obj.getMountedImage(0)) && %obj.getMountedImage(0).getID() == SpyFakeDeathWatchImage.getID())
      {
         if(%obj.getImageState(0) $= "Ready" && %obj.cloakEnergy == 100 && !%obj.isCloaked)
         {
            %amm*=0.1;
            
            %ok = (%obj.getDamageLevel() + %amm < %this.maxDamage);
            
            if(%ok)
            {
               %watch = SpyFakeDeathWatchImage.getID();
               
               %obj.cloakWatch = %watch;
               %obj.setImageAmmo(0,1);
               
               %obj.isCloaking = 0;
               %obj.isCloaked = 1;
               %obj.cloakFakeDeathTime = getSimTime();
               %obj.hideNode("ALL");
               %obj.setShapeNameDistance(10);
               
               if(%source.client == %obj.client)
                  %obj.cloakFakeKiller = "SELF";
               else if(isObject(%source.client))
                  %obj.cloakFakeKiller = %source.client;
               else
                  %obj.cloakFakeKiller = "";
               
               %obj.playDeathCry();
               
               %bot = new AIPlayer()
               {
                  dataBlock = %obj.dataBlock;
               };MissionCleanup.add(%bot);
               
               %bot.setTransform(%obj.getTransform());
               %bot.setVelocity(%obj.getVelocity());
               
               //Emotes e.g. burning effects
               if(isObject(%obj.getMountedImage(3)))
               {
                  %bot.mountImage(%obj.getMountedImage(3),3);
                  %obj.unMountImage(3);
               }
               
               if(isObject(%obj.client))
               {
                  %client = %obj.client;
                  %client.play2d(%watch.cloakSound);
                  
                  %client.player = %bot;
                  %client.applyBodyParts();
                  %client.applyBodyColors();
                  %client.ignoreNextUnuse = 1;
                  if(%client.minigame.tdmLivesLimit)
                  {
                     %client.tdmLives++;
                  }
                  %client.onDeath(%source, %source ? %source.client : 0, %type, "Body");
                  %bot.kill();
                  %client.player = %obj;
                  %client.setControlObject(%obj);
                  messageClient(%client,'MsgYourSpawn');
               }
            }
         }
         else if(%obj.isCloaked)
         {
            %amm*=0.1;
         }
      }
      Parent::Damage(%this,%obj,%source,%pos,%amm,%type);
   }
   
   function GameConnection::onDeath(%this,%sourceObject,%sourceClient,%damageType,%damageLoc)
   {
      Parent::onDeath(%this,%sourceObject,%sourceClient,%damageType,%damageLoc);
      if(isObject(%sourceClient.player) && %sourceClient.player.cloakFakeKiller == %this && getSimTime() - %sourceClient.player.cloakFakeDeathTime < 15000)
      {
         if(isObject(%this.minigame))
            %this.minigame.messageAll('',"\c3" @ %sourceClient.getPlayerName() @ " <bitmap:base/client/ui/ci/star> \c0got REVENGE on \c3" @ %this.getPlayerName() @ "\c0!");
         
         %this.play2d(spyWarpCloakRevengeSound);
         %sourceClient.play2d(spyWarpCloakRevengeSound);
      }
      if(isObject(%this.minigame))
         for(%i=0;%i<%this.minigame.numMembers;%i++)
         {
            %m = %this.minigame.member[%i];
            if(%m.player.cloakFakeKiller == %this)
               %m.player.cloakFakeKiller = "";
         }
   }
   
   function servercmdusetool(%client,%slot)
   {
      //if(isObject(%client.player))
      //{
      //   if(isEventPending(%client.player.cloakDeathWatchSched))
      //      cancel(%client.player.cloakDeathWatchSched);
      //   
      //   if(isObject(%client.player.tool[%slot]) && %client.player.tool[%slot].getID() == SpyFakeDeathWatchItem.getID() && %client.player.cloakEnergy < 100)
      //   {
      //      %client.player.cloakDeathWatchSched = schedule(100,0,servercmdUseTool,%client,%slot);
      //      return;
      //   }
      //}
      Parent::servercmdusetool(%client,%slot);
   }
   
   function servercmdUnUseTool(%client)
   {
      if(%client.ignoreNextUnuse)
      {
         %client.ignoreNextUnuse = 0;
         return;
      }
      
      if(isEventPending(%client.player.cloakDeathWatchSched))
         cancel(%client.player.cloakDeathWatchSched);
      
      Parent::servercmdUnUseTool(%client);
   }
   
   function Player::endCloak(%player)
   {
      if(!%player.isCloaked || %player.isCloaking)
         return;
      
      if(%player.cloakWatch == SpyFakeDeathWatchImage.getID())
      {
         %en1 = %player.cloakEnergy - (getSimTime()-%player.cloakFakeDeathTime)/80;
         %en2 = %en1 - 40;
         if(%en2 < 0)
            %player.cloakEnergy = 0;
         else
            %player.cloakEnergy = %en2;
         
         if(isObject(%player.client))
         {
            %a = %en2;
            %b = %en1;
            %c = 100;
            %end1 = (%a / %c) * 25;
            %end2 = (%b / %c) * 25;
            %start = "<color:FFFFFF>";
            %sep1 = "<color:FF0000>";
            %sep2 = "<color:111111>";
            %done1 = (%end1 == 0);
            %done2 = (%end2 == 0);
            if(%done2)
               %str = %sep2;
            else if(%done1)
               %str = %sep1;
            else
               %str = %start;
            
            for(%i=0;%i<25;%i++)
            {
               if(%i > %end1 && !%done1)
               {
                  %str = %str @ %sep1;
                  %done1 = 1;
               }
               if(%i > %end2 && !%done2)
               {
                  %str = %str @ %sep2;
                  %done2 = 1;
               }
               %str = %str @ "|";
            }
            bottomprint(%player.client, "<just:center>" @ %str, 2, 1);
         }
         
         if(%player.cloakFakeKiller $= "SELF")
            %player.client.incScore(-%player.client.minigame.Points_KillSelf);
         else if(isObject(%player.cloakFakeKiller))
            %player.cloakFakeKiller.incScore(-%player.cloakFakeKiller.minigame.Points_KillPlayer);
         
         %player.client.incScore(-%player.client.minigame.Points_Die);
      }
      
      Parent::endCloak(%player);
   }
   
   function MinigameSO::getForceRespawn(%mini,%client)
   {
      if(%client.ignoreNextUnuse)
         return 0;
      
      return Parent::getForceRespawn(%mini,%client);
   }
};activatePackage(FakeDeathWatch);