datablock AudioDescription(AudioClosestQuiet3d : AudioClosest3d)
{
	volume = 0.75;
};

datablock AudioProfile(spyCloakSound)
{
	filename = "./playerCloak.wav";
	description = AudioClosestQuiet3d;
	preload = true;
};

datablock AudioProfile(spyUncloakSound)
{
	filename = "./playerUncloak.wav";
	description = AudioClosestQuiet3d;
	preload = true;
};

datablock ItemData(SpyWatchItem)
{
	category = "Weapon";  // Mission editor category
	className = "Weapon"; // For inventory system

	 // Basic Item Properties
	shapeFile = "./SpyWatchItem.dts";
	rotate = false;
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;

	//gui stuff
	uiName = "Invisibility Watch";
	iconName = "./icon_iwatch";
	doColorShift = true;
	colorShiftColor = "0.6 0.6 0.6 1";

	 // Dynamic properties defined by the scripts
	image = SpyWatchImage;
	canDrop = true;
	
	variableStore = "cloakEnergy";
	variableInitial = 100;
	variableReplace = 1;
};

datablock ShapeBaseImageData(SpyWatchImage)
{
   // Basic Item properties
   shapeFile = "./SpyWatchHand.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 -1000";
   eyeOffset = "0.8 0.65 -0.4";
   rotation = eulerToMatrix( "0 0 0" );
   eyeRotation = eulerToMatrix( "10 0 0" );

   className = "WeaponImage";
   item = SpyWatchItem;

   //raise your arm up or not
   armReady = false;
   
   //cloak stats
   isCloakWatch = 1;
   minCloakEnergy = 15;
   cloakChargeRate = 0.1;
   cloakDamageFlicker = 1;
   cloakKillRegen = 40;
   
   cloakSound = spyCloakSound;
   uncloakSound = spyUncloakSound;

   doColorShift = true;
   colorShiftColor = SpyWatchItem.colorShiftColor;

   // Initial start up state
   stateName[0]                     = "Activate";
   stateTimeoutValue[0]             = 0.2;
   stateTransitionOnTimeout[0]      = "CloakUse";
   stateSound[0]                    = weaponSwitchSound;
   
   stateName[1]                     = "CloakUse";
   stateScript[1]                   = "onCloakUse";
   stateTransitionOnAmmo[1]         = "Cloaking";
   stateTransitionOnTimeout[1]      = "CloakUse";
   stateWaitForTimeout[1]           = 0;
   stateTimeoutValue[1]             = "0.05";
   stateAllowImageChange[1]         = true;
   stateSequence[1]                 = "Ready";
   
   stateName[2]                     = "Cloaking";
   stateTransitionOnNoAmmo[2]       = "Uncloak";
   stateAllowImageChange[2]         = true;
   stateSequence[2]                 = "Cloaking";
   stateTimeoutValue[2]             = 1;
   stateTransitionOnTimeout[2]      = "Cloaked";
   
   stateName[3]                     = "Cloaked";
   stateTransitionOnNoAmmo[3]       = "Uncloak";
   stateTransitionOnTimeout[3]      = "Cloaked";
   stateTimeoutValue[3]             = 0.05;
   stateWaitForTimeout[3]           = false;
   stateScript[3]                   = "onCloakCheck";
   stateAllowImageChange[3]         = true;
   stateSequence[3]                 = "FullyCloaked";
   
   stateName[4]                     = "Uncloak";
   stateTransitionOnAmmo[4]         = "Cloaking";
   stateAllowImageChange[4]         = true;
   stateSequence[4]                 = "Uncloaking";
   stateTimeoutValue[4]             = 1;
   stateTransitionOnTimeout[4]      = "CloakUse";
};

function SpyWatchImage::onMount(%this,%obj,%slot)
{
   Parent::onMount(%this,%obj,%slot);
   
   %obj.setImageAmmo(%slot,0);
   
   %obj.cloakEnergy = %obj.cloakEnergy + 20*%this.cloakChargeRate*(getSimTime()-%obj.lastUnCloakTime)/1000;
   
   if(%obj.cloakEnergy > 100) %obj.cloakEnergy = 100;
   if(%obj.cloakEnergy < 0) %obj.cloakEnergy = 0;
   
   if(isObject(%obj.client))
   {
      %a = %obj.cloakEnergy;
      %b = 100;
      %start = "<color:FFFFFF>";
      %sep = "<color:111111>";
      %done = (%a >= %b) || (%a == 0);
      %end = (%a / %b) * 25;
      %str = (%a == 0 ? %sep : %start);
      
      for(%i=0;%i<25;%i++)
      {
         if(%i > %end && !%done)
         {
            %str = %str @ %sep;
            %done = 1;
         }
         %str = %str @ "|";
      }
      bottomprint(%obj.client, "<just:center>" @ %str, 2, 1);
   }
}

function SpyWatchImage::onCloakUse(%this,%obj,%slot)
{
   //put arm down since it's invisible from outside
   fixArmReady(%obj);
   
   if(!%obj.isCloaking)
   {
      %obj.cloakEnergy = %obj.cloakEnergy + 20*%this.cloakChargeRate*(getSimTime()-%obj.lastUnCloakTime)/1000;
      %obj.lastUnCloakTime = getSimTime();
   }
   
   if(%obj.cloakEnergy > 100) %obj.cloakEnergy = 100;
   if(%obj.cloakEnergy < 0) %obj.cloakEnergy = 0;
   
   if(isObject(%obj.client))
   {
      %a = %obj.cloakEnergy;
      %b = 100;
      %start = "<color:FFFFFF>";
      %sep = "<color:111111>";
      %done = (%a >= %b) || (%a == 0);
      %end = (%a / %b) * 25;
      %str = (%a == 0 ? %sep : %start);
      
      for(%i=0;%i<25;%i++)
      {
         if(%i > %end && !%done)
         {
            %str = %str @ %sep;
            %done = 1;
         }
         %str = %str @ "|";
      }
      bottomprint(%obj.client, "<just:center>" @ %str, 2, 1);
   }
   
   if(!%obj.isCloaked && !%obj.isCloaking && %obj.cloakEnergy >= %this.minCloakEnergy)
      %obj.beginCloak(%this);
}

function SpyWatchImage::onCloakCheck(%this,%obj,%slot)
{
   if(%obj.getState() $= "Dead")
      return;
   
   %obj.cloakEnergy -= 0.5;
   if(%obj.cloakEnergy <= 0)
   {
      %obj.cloakEnergy = 0;
      %obj.endCloak();
   }
   
   if(isObject(%obj.client))
   {
      %a = %obj.cloakEnergy;
      %b = 100;
      %start = "<color:FFFFFF>";
      %sep = "<color:111111>";
      %done = (%a >= %b) || (%a == 0);
      %end = (%a / %b) * 25;
      %str = (%a == 0 ? %sep : %start);
      
      for(%i=0;%i<25;%i++)
      {
         if(%i > %end && !%done)
         {
            %str = %str @ %sep;
            %done = 1;
         }
         %str = %str @ "|";
      }
      bottomprint(%obj.client, "<just:center>" @ %str, 2, 1);
   }
}